@@ -68,7 +68,7 @@ bool TypeInteger::ok(const std::string& value, long& n) const {
     return !range_ || (n >= range_->lower_ && n <= range_->upper_);
 }
 
-bool TypeInteger::expand(const MarsExpandContext& ctx, std::string& value) const {
+bool TypeInteger::expand(const MarsExpandContext& ctx, std::string& value, const MarsRequest& /* request */) const {
     long n = 0;
     if (ok(value, n)) {
         static eckit::Translator<long, std::string> l2s;
